void emit_battery(global_t *globals, char *header_shown);
void emit_acstate(global_t *globals, char *header_shown);
void emit_zone(global_t *globals, char *header_shown);
void emit_fan(global_t *globals, char *header_shown);
